@@ -2,23 +2,20 @@
 """Page Admin module."""
 from pages import settings
 from pages.models import Page, Content, PageAlias
-from pages.http import get_language_from_request, get_template_from_request
+from pages.phttp import get_language_from_request, get_template_from_request
 from pages.utils import get_placeholders
 from pages.templatetags.pages_tags import PlaceholderNode
-from pages.admin.utils import get_connected, make_inline_admin
-from pages.admin.forms import PageForm
+from pages.admin.forms import make_form
 from pages.admin.views import traduction, get_content, sub_menu
 from pages.admin.views import list_pages_ajax
 from pages.admin.views import change_status, modify_content, delete_content
 from pages.admin.views import move_page
-from pages.admin.actions import export_pages_as_json, import_pages_from_json
 from pages.permissions import PagePermission
-from pages.http import auto_render
 import pages.widgets
 
 from django.contrib import admin
 from django.utils.translation import ugettext as _, ugettext_lazy
-from django.utils.encoding import force_unicode
+from django.utils.encoding import force_text
 from django.conf import settings as global_settings
 from django.http import HttpResponseRedirect, Http404
 from django.contrib.admin.util import unquote
@@ -31,11 +28,39 @@ else:
 from os.path import join
 
 
+from django.db import models
+def create_page_model(placeholders=[]):
+    """
+    Create Page model
+    """
+    app_label='pages'
+    module = 'pages.models.test'
+    class Meta:
+        # Using type('Meta', ...) gives a dictproxy error during model creation
+        pass
+
+    # app_label must be set using the Meta inner class
+    setattr(Meta, 'app_label', app_label)
+
+    # Set up a dictionary to simulate declarations within a class
+    attrs = {'__module__': module, 'Meta': Meta}
+
+    # Add in any fields that were provided
+    for p in placeholders:
+        attrs[p.name] = models.TextField(blank=True)
+    
+    attrs["slug"] = models.TextField()
+    attrs["title"] = models.TextField()
+
+    # Create the class, which automatically triggers ModelBase processing
+    model = type("Page", (Page,), attrs)
+
+    return model
+
+
 class PageAdmin(admin.ModelAdmin):
     """Page Admin class."""
 
-    form = PageForm
-    exclude = ['author', 'parent']
     # these mandatory fields are not versioned
     mandatory_placeholders = ('title', 'slug')
     general_fields = ['title', 'slug', 'status', 'target',
@@ -73,9 +98,6 @@ class PageAdmin(admin.ModelAdmin):
         }],
     )
 
-    if settings.PAGE_EXPORT_ENABLED:
-        actions = [export_pages_as_json]
-
     class Media:
         css = {
             'all': [join(settings.PAGES_MEDIA_URL, path) for path in (
@@ -92,8 +114,13 @@ class PageAdmin(admin.ModelAdmin):
             'javascript/jquery.query-2.1.7.js',
         )]
 
+    @classmethod
+    def add_action(cls, method):
+        if method not in cls.actions:
+            cls.actions.append(method)
+
     def urls(self):
-        from django.conf.urls.defaults import patterns, url, include
+        from django.conf.urls import patterns, url, include
 
         # Admin-site-wide views.
         urlpatterns = patterns('',
@@ -113,12 +140,16 @@ class PageAdmin(admin.ModelAdmin):
             url(r'^(?P<page_id>[0-9]+)/change-status/$',
                 change_status, name='page-change-status'),
         )
-        if settings.PAGE_IMPORT_ENABLED:
-            urlpatterns += patterns('',
-                url(r'^import-json/$',
-                    self.import_pages, name='import-pages-from-json'),
-            )
-
+        
+        """for app_name in global_settings.INSTALLED_APPS:
+            try:
+                module_ = __import__(app_name, globals(), locals(), ['object'], -1)
+            except ImportError:
+                continue
+            if hasattr(module_, "PAGE_ADMIN_URLS"):
+                urls = __import__(getattr(module_, "PAGE_ADMIN_URLS"), globals(), locals(), ['object'], -1)
+                urlpatterns += urls.urlpatterns"""
+                
         urlpatterns += super(PageAdmin, self).urls
 
         return urlpatterns
@@ -220,8 +251,14 @@ class PageAdmin(admin.ModelAdmin):
         """Get a :class:`Page <pages.admin.forms.PageForm>` for the
         :class:`Page <pages.models.Page>` and modify its fields depending on
         the request."""
-        form = super(PageAdmin, self).get_form(request, obj, **kwargs)
+        
+        template = get_template_from_request(request, obj)
+        
+        model = create_page_model(get_placeholders(template))
 
+        form = make_form(model)
+
+        # bound the form
         language = get_language_from_request(request)
         form.base_fields['language'].initial = language
         if obj:
@@ -238,7 +275,7 @@ class PageAdmin(admin.ModelAdmin):
             template_choices.insert(0, (settings.PAGE_DEFAULT_TEMPLATE,
                     _('Default template')))
             form.base_fields['template'].choices = template_choices
-            form.base_fields['template'].initial = force_unicode(template)
+            form.base_fields['template'].initial = force_text(template)
 
         for placeholder in get_placeholders(template):
             name = placeholder.name
@@ -257,8 +294,6 @@ class PageAdmin(admin.ModelAdmin):
         language = get_language_from_request(request)
         extra_context = {
             'language': language,
-            # don't see where it's used
-            #'lang': current_lang,
             'page_languages': settings.PAGE_LANGUAGES,
         }
         try:
@@ -281,8 +316,21 @@ class PageAdmin(admin.ModelAdmin):
                                     l[0], "title") and l[0] != language]
         extra_context['page'] = obj
 
-        return super(PageAdmin, self).change_view(request, object_id,
+        response = super(PageAdmin, self).change_view(request, object_id,
             form_url=form_url, extra_context=extra_context)
+        if request.method == 'POST' and isinstance(response, HttpResponseRedirect):
+            if '_continue' in request.POST or '_saveasnew' in request.POST or '_addanother' in request.POST:
+                addlanguage = True
+            else:
+                addlanguage = False
+            if addlanguage:
+                from six.moves import urllib
+                splitted = list(urllib.parse.urlparse(response.url))
+                query = urllib.parse.parse_qs(splitted[4])
+                query['language'] = language
+                splitted[4] = urllib.parse.urlencode(query)
+                response = HttpResponseRedirect(urllib.parse.urlunparse(splitted))
+        return response
 
 
     def add_view(self, request, form_url='', extra_context=None):
@@ -291,8 +339,6 @@ class PageAdmin(admin.ModelAdmin):
             'language': get_language_from_request(request),
             'page_languages': settings.PAGE_LANGUAGES,
         }
-        template = get_template_from_request(request)
-        #extra_context['placeholders'] = get_placeholders(template)
         return super(PageAdmin, self).add_view(request, form_url,
                                                             extra_context)
 
@@ -348,43 +394,6 @@ class PageAdmin(admin.ModelAdmin):
 
         return change_list
 
-    def import_pages(self, request):
-        if not self.has_add_permission(request):
-            return admin.site.login(request)
-
-        return import_pages_from_json(request)
-
-
-class PageAdminWithDefaultContent(PageAdmin):
-    """
-    Fill in values for content blocks from official language
-    if creating a new translation
-    """
-    def get_form(self, request, obj=None, **kwargs):
-        form = super(PageAdminWithDefaultContent, self
-            ).get_form(request, obj, **kwargs)
-
-        language = get_language_from_request(request)
-
-        if global_settings.LANGUAGE_CODE == language:
-            # this is the "official" language
-            return form
-
-        if Content.objects.filter(page=obj, language=language).count():
-            return form
-
-        # this is a new page, try to find some default content
-        template = get_template_from_request(request, obj)
-        for placeholder in get_placeholders(template):
-            name = placeholder.name
-            form.base_fields[name] = placeholder.get_field(obj, language,
-                initial=Content.objects.get_content(obj,
-                    global_settings.LANGUAGE_CODE, name))
-        return form
-
-
-for admin_class, model, options in get_connected():
-    PageAdmin.inlines.append(make_inline_admin(admin_class, model, options))
 
 try:
     admin.site.register(Page, PageAdmin)
@@ -401,7 +410,7 @@ class ContentAdmin(admin.ModelAdmin):
 
 class AliasAdmin(admin.ModelAdmin):
     list_display = ('page', 'url',)
-    list_editable = ('url', )
+    list_editable = ('url',)
 
 try:
     admin.site.register(PageAlias, AliasAdmin)
