@@ -2,13 +2,13 @@
 """Django page CMS ``managers``."""
 from pages import settings
 from pages.cache import cache
-from pages.utils import normalize_url
-from pages.http import get_slug
+from pages.utils import normalize_url, get_now
+from pages.phttp import get_slug
 
 from django.db import models, connection
 from django.db.models import Q
 from django.core.exceptions import ObjectDoesNotExist
-from django.contrib.auth.models import User, SiteProfileNotAvailable
+from django.contrib.auth.models import User
 from django.db.models import Avg, Max, Min, Count
 from django.contrib.sites.models import Site
 from django.conf import settings as global_settings
@@ -16,14 +16,6 @@ from django.utils.translation import ugettext_lazy as _
 
 from mptt.managers import TreeManager
 
-from datetime import datetime
-
-ISODATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f' # for parsing dates from JSON
-
-# This affects how content is saved
-CATEGORY_INSTALLED = 'pages.plugins.category' in global_settings.INSTALLED_APPS
-CATEGORY_TYPE = 'category'
-
 
 class PageManager(TreeManager):
     """
@@ -89,11 +81,11 @@ class PageManager(TreeManager):
         queryset = queryset.filter(status=self.model.PUBLISHED)
 
         if settings.PAGE_SHOW_START_DATE:
-            queryset = queryset.filter(publication_date__lte=datetime.now())
+            queryset = queryset.filter(publication_date__lte=get_now())
 
         if settings.PAGE_SHOW_END_DATE:
             queryset = queryset.filter(
-                Q(publication_end_date__gt=datetime.now()) |
+                Q(publication_end_date__gt=get_now()) |
                 Q(publication_end_date__isnull=True)
             )
 
@@ -109,14 +101,14 @@ class PageManager(TreeManager):
         :attr:`Page.publication_date`."""
         pub = self.on_site().filter(status=self.model.DRAFT)
         if settings.PAGE_SHOW_START_DATE:
-            pub = pub.filter(publication_date__gte=datetime.now())
+            pub = pub.filter(publication_date__gte=get_now())
         return pub
 
     def expired(self):
         """Creates a :class:`QuerySet` of expired using the page's
         :attr:`Page.publication_end_date`."""
         return self.on_site().filter(
-            publication_end_date__lte=datetime.now())
+            publication_end_date__lte=get_now())
 
     def from_path(self, complete_path, lang, exclude_drafts=True):
         """Return a :class:`Page <pages.models.Page>` according to
@@ -149,135 +141,13 @@ class PageManager(TreeManager):
                 if page.get_complete_slug(lang) == complete_path:
                     return page
         return None
-
-    def create_and_update_from_json_data(self, d, user):
-        """
-        Create or update page based on python dict d loaded from JSON data.
-        This applies all data except for redirect_to, which is done in a
-        second pass after all pages have been imported,
-
-        user is the User instance that will be used if the author can't
-        be found in the DB.
-
-        returns (page object, created, messages).
-
-        created is True if this was a new page or False if an existing page
-        was updated.
-
-        messages is a list of strings warnings/messages about this import
-        """
-        page = None
-        parent = None
-        parent_required = True
-        created = False
-        messages = []
-
-        page_languages = set(lang[0] for lang in settings.PAGE_LANGUAGES)
-
-        for lang, s in d['complete_slug'].items():
-            if lang not in page_languages:
-                messages.append(_("Language '%s' not imported") % (lang,))
-                continue
-
-            page = self.from_path(s, lang, exclude_drafts=False)
-            if page and page.get_complete_slug(lang) == s:
-                break
-            if parent_required and parent is None:
-                if '/' in s:
-                    parent = self.from_path(s.rsplit('/', 1)[0], lang,
-                        exclude_drafts=False)
-                else:
-                    parent_required = False
-        else:
-            # can't find an existing match, need to create a new Page
-            page = self.model(parent=parent)
-            created = True
-
-        def custom_get_user_by_email(email):
-            """
-            Allow the user profile class to look up a user by email
-            address
-            """
-            # bit of an unpleasant hack that requres the logged-in
-            # user has a profile, but I don't want to reproduce the
-            # code in get_profile() here
-            try:
-                profile = user.get_profile()
-            except (SiteProfileNotAvailable, ObjectDoesNotExist):
-                return User.objects.get(email=email)
-            get_user_by_email = getattr(profile, 'get_user_by_email', None)
-            if get_user_by_email:
-                return get_user_by_email(email)
-            return User.objects.get(email=email)
-
-        try:
-            page.author = custom_get_user_by_email(d['author_email'])
-        except (User.DoesNotExist, User.MultipleObjectsReturned):
-            page.author = user
-            messages.append(_("Original author '%s' not found")
-                % (d['author_email'],))
-
-        page.creation_date = datetime.strptime(d['creation_date'],
-            ISODATE_FORMAT)
-        page.publication_date = datetime.strptime(d['publication_date'],
-            ISODATE_FORMAT) if d['publication_date'] else None
-        page.publication_end_date = datetime.strptime(d['publication_end_date'],
-            ISODATE_FORMAT) if d['publication_end_date'] else None
-        page.last_modification_date = datetime.strptime(
-            d['last_modification_date'], ISODATE_FORMAT)
-        page.status = {
-            'published': self.model.PUBLISHED,
-            'hidden': self.model.HIDDEN,
-            'draft': self.model.DRAFT,
-            }[d['status']]
-        page.template = d['template']
-        page.redirect_to_url = d['redirect_to_url']
-
-        page.save()
-
-        if settings.PAGE_USE_SITE_ID:
-            if d['sites']:
-                for site in d['sites']:
-                    try:
-                        page.sites.add(Site.objects.get(domain=site))
-                    except Site.DoesNotExist:
-                        messages.append(_("Could not add site '%s' to page")
-                            % (site,))
-            if not settings.PAGE_HIDE_SITES and not page.sites.count():
-                # need at least one site
-                page.sites.add(Site.objects.get(pk=global_settings.SITE_ID))
-
-        from pages.models import Content
-        def create_content(lang, ctype, body):
-            Content.objects.create_content_if_changed(page, lang, ctype, body)
-
-        for lang in d['content_language_updated_order']:
-            if lang not in page_languages:
-                continue
-            create_content(lang, 'slug',
-                d['complete_slug'][lang].rsplit('/', 1)[-1])
-            create_content(lang, 'title', d['title'][lang])
-            for ctype, langs_bodies in d['content'].items():
-                create_content(lang, ctype, langs_bodies[lang])
-
-        return page, created, messages
-
-
+        
 
 class ContentManager(models.Manager):
     """:class:`Content <pages.models.Content>` manager methods"""
 
     PAGE_CONTENT_DICT_KEY = "page_content_dict_%d_%s_%d"
 
-    def sanitize(self, content):
-        """Sanitize a string in order to avoid possible XSS using
-        ``html5lib``."""
-        import html5lib
-        from html5lib import sanitizer
-        p = html5lib.HTMLParser(tokenizer=sanitizer.HTMLSanitizer)
-        dom_tree = p.parseFragment(content)
-        return dom_tree.toxml()
-
     def set_or_create_content(self, page, language, ctype, body):
         """Set or create a :class:`Content <pages.models.Content>` for a
         particular page and language.
@@ -287,8 +157,6 @@ class ContentManager(models.Manager):
         :param ctype: the content type.
         :param body: the content of the Content object.
         """
-        if settings.PAGE_SANITIZE_USER_INPUT:
-            body = self.sanitize(body)
         try:
             content = self.filter(page=page, language=language,
                                   type=ctype).latest('creation_date')
@@ -309,8 +177,6 @@ class ContentManager(models.Manager):
         :param ctype: the content type.
         :param body: the content of the Content object.
         """
-        if settings.PAGE_SANITIZE_USER_INPUT:
-            body = self.sanitize(body)
         try:
             content = self.filter(page=page, language=language,
                                   type=ctype).latest('creation_date')
@@ -329,12 +195,6 @@ class ContentManager(models.Manager):
             for c in oldest_content:
                 c.delete()
 
-        # Delete also old category if relevant
-        if CATEGORY_INSTALLED and ctype == CATEGORY_TYPE:
-            cat = self.filter(page=page, language=language, type=ctype)
-            cat = cat.exclude(pk=content.pk)
-            cat.delete()
-
         return content
 
     def get_content_object(self, page, language, ctype):
@@ -347,7 +207,7 @@ class ContentManager(models.Manager):
         }
         if page.freeze_date:
             params['creation_date__lte'] = page.freeze_date
-        return  self.filter(**params).latest()
+        return self.filter(**params).latest()
 
     def get_content(self, page, language, ctype, language_fallback=False):
         """Gets the latest content string for a particular page, language and
