@@ -3,7 +3,7 @@
 from pages.cache import cache
 from pages.utils import get_placeholders, normalize_url, get_now
 from pages.managers import PageManager, ContentManager
-from pages.managers import PageAliasManager, ISODATE_FORMAT
+from pages.managers import PageAliasManager
 from pages import settings
 
 from datetime import datetime
@@ -17,193 +17,16 @@ from django.core.urlresolvers import reverse
 from django.core.exceptions import ObjectDoesNotExist
 from django.contrib.sites.models import Site
 from django.conf import settings as global_settings
+from django.utils.encoding import python_2_unicode_compatible
 
 
 from mptt.models import MPTTModel
-if settings.PAGE_TAGGING:
-    from taggit.managers import TaggableManager
 
 PAGE_CONTENT_DICT_KEY = ContentManager.PAGE_CONTENT_DICT_KEY
 
 
-class TitleSlugMixin(models.Model):
-    """Anything with a slug based on a title
-    """
-
-    class Meta:
-        abstract = True
-
-    def __init__(self, *args, **kwargs):
-        """Constructions specific to this mixin"""
-        self._complete_slug = None
-
-        return super(TitleSlugMixin, self).__init__(*args, **kwargs)
-
-    def get_url_path(self, language=None):
-        """Return the URL's path component. Add the language prefix if
-        ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to ``True``.
-
-        :param language: the wanted url language.
-        """
-        if self.is_first_root():
-            # this is used to allow users to change URL of the root
-            # page. The language prefix is not usable here.
-            try:
-                return reverse('pages-root')
-            except Exception:
-                pass
-        url = self.get_complete_slug(language)
-        if not language:
-            language = settings.PAGE_DEFAULT_LANGUAGE
-        if settings.PAGE_USE_LANGUAGE_PREFIX:
-            return reverse('pages-details-by-path',
-                args=[language, url])
-        else:
-            return reverse('pages-details-by-path', args=[url])
-
-    def get_absolute_url(self, language=None):
-        """Alias for `get_url_path`.
-
-        This method is only there for backward compatibility and will be
-        removed in a near futur.
-
-        :param language: the wanted url language.
-        """
-        return self.get_url_path(language=language)
-
-    def get_complete_slug(self, language=None, hideroot=True):
-        """Return the complete slug of this page by concatenating
-        all parent's slugs.
-
-        :param language: the wanted slug language."""
-        if not language:
-            language = settings.PAGE_DEFAULT_LANGUAGE
-
-        if self._complete_slug and language in self._complete_slug:
-            return self._complete_slug[language]
-
-        self._complete_slug = cache.get(self.PAGE_URL_KEY % (self.id))
-        if self._complete_slug is None:
-            self._complete_slug = {}
-        elif language in self._complete_slug:
-            return self._complete_slug[language]
-
-        if hideroot and settings.PAGE_HIDE_ROOT_SLUG and self.is_first_root():
-            url = u''
-        else:
-            url = u'%s' % self.slug(language)
-
-        key = self.ANCESTORS_KEY % self.id
-        ancestors = cache.get(key, None)
-        if ancestors is None:
-            ancestors = self.get_ancestors(ascending=True)
-            cache.set(key, ancestors)
-
-        for ancestor in ancestors:
-            url = ancestor.slug(language) + u'/' + url
-
-        self._complete_slug[language] = url
-        cache.set(self.PAGE_URL_KEY % (self.id), self._complete_slug)
-        return url
-
-    def get_url(self, language=None):
-        """Alias for `get_complete_slug`.
-
-        This method is only there for backward compatibility and will be
-        removed in a near futur.
-
-        :param language: the wanted url language.
-        """
-        return self.get_complete_slug(language=language)
-
-    def slug_with_level(self, language=None):
-        """Display the slug of the page prepended with insecable
-        spaces equal to simluate the level of page in the hierarchy."""
-        level = ''
-        if self.level:
-            for n in range(0, self.level):
-                level += '&nbsp;&nbsp;&nbsp;'
-        return mark_safe(level + self.slug(language))
-
-    def slug(self, language=None, fallback=True):
-        """
-        Return the slug of the page depending on the given language.
-
-        :param language: wanted language, if not defined default is used.
-        :param fallback: if ``True``, the slug will also be searched in other \
-        languages.
-        """
-
-        slug = self.get_content(language, 'slug', language_fallback=fallback)
-
-        return slug
-
-    def title(self, language=None, fallback=True):
-        """
-        Return the title of the page depending on the given language.
-
-        :param language: wanted language, if not defined default is used.
-        :param fallback: if ``True``, the slug will also be searched in \
-        other languages.
-        """
-        if not language:
-            language = settings.PAGE_DEFAULT_LANGUAGE
-
-        return self.get_content(language, 'title', language_fallback=fallback)
-
-
-class HasContentMixin(models.Model):
-    """Anything with content in a :class:`<pages.models.Content>`
-    """
-
-    class Meta:
-        abstract = True
-
-    def get_content(self, language, ctype, language_fallback=False):
-        """Shortcut method for retrieving a piece of page content
-
-        :param language: wanted language, if not defined default is used.
-        :param ctype: the type of content.
-        :param fallback: if ``True``, the content will also be searched in \
-        other languages.
-        """
-        return Content.objects.get_content(self, language, ctype,
-            language_fallback)
-
-    def expose_content(self):
-        """Return all the current content of this page into a `string`.
-
-        This is used by the haystack framework to build the search index."""
-        placeholders = get_placeholders(self.get_template())
-        exposed_content = []
-        for lang in self.get_languages():
-            for ctype in [p.name for p in placeholders]:
-                content = self.get_content(lang, ctype, False)
-                if content:
-                    exposed_content.append(content)
-        return u"\r\n".join(exposed_content)
-
-    def content_by_language(self, language):
-        """
-        Return a list of latest published
-        :class:`Content <pages.models.Content>`
-        for a particluar language.
-
-        :param language: wanted language,
-        """
-        placeholders = get_placeholders(self.get_template())
-        content_list = []
-        for ctype in [p.name for p in placeholders]:
-            try:
-                content = Content.objects.get_content_object(self,
-                    language, ctype)
-                content_list.append(content)
-            except Content.DoesNotExist:
-                pass
-        return content_list
-
-
-class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
+@python_2_unicode_compatible
+class Page(MPTTModel):
     """
     This model contain the status, dates, author, template.
     The real content of the page can be found in the
@@ -243,7 +66,6 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
 
     PAGE_LANGUAGES_KEY = "page_%d_languages"
     PAGE_URL_KEY = "page_%d_url"
-    PAGE_BROKEN_LINK_KEY = "page_broken_link_%s"
     ANCESTORS_KEY = 'ancestors_%d'
     CHILDREN_KEY = 'children_%d'
     PUB_CHILDREN_KEY = 'pub_children_%d'
@@ -289,7 +111,7 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
     objects = PageManager()
 
     if settings.PAGE_TAGGING:
-        tags = TaggableManager(blank=True)
+        tags = settings.PAGE_TAGGING_FIELD()
 
     class Meta:
         """Make sure the default page ordering is correct."""
@@ -305,6 +127,7 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
         self._languages = None
         self._content_dict = None
         self._is_first_root = None
+        self._complete_slug = None
         super(Page, self).__init__(*args, **kwargs)
 
     def save(self, *args, **kwargs):
@@ -323,8 +146,6 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
             else:
                 self.publication_date = None
         self.last_modification_date = get_now()
-        # let's assume there is no more broken links after a save
-        cache.delete(self.PAGE_BROKEN_LINK_KEY % self.id)
         super(Page, self).save(*args, **kwargs)
         # fix sites many-to-many link when the're hidden from the form
         if settings.PAGE_HIDE_SITES and self.sites.count() == 0:
@@ -371,22 +192,18 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
 
     def get_children_for_frontend(self):
         """Return a :class:`QuerySet` of published children page"""
-        key = self.PUB_CHILDREN_KEY % self.id
-        children = cache.get(key, None)
-        if children is None:
-            children = Page.objects.filter_published(self.get_children())
-            cache.set(key, children)
-        return children
+        return self.published_children()
 
     def get_date_ordered_children_for_frontend(self):
         """Return a :class:`QuerySet` of published children page ordered
         by publication date."""
-        return self.get_children_for_frontend().order_by('-publication_date')
+        return self.published_children().order_by('-publication_date')
 
     def move_to(self, target, position='first-child'):
         """Invalidate cache when moving"""
 
-        # Invalidate both in case position matters, otherwise only target is needed
+        # Invalidate both in case position matters, 
+        # otherwise only target is needed.
         self.invalidate()
         target.invalidate()
         super(Page, self).move_to(target, position=position)
@@ -499,13 +316,6 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
                 return t[1]
         return template
 
-    def has_broken_link(self):
-        """
-        Return ``True`` if the page have broken links to other pages
-        into the content.
-        """
-        return cache.get(self.PAGE_BROKEN_LINK_KEY % self.id)
-
     def valid_targets(self):
         """Return a :class:`QuerySet` of valid targets for moving a page
         into the tree.
@@ -516,117 +326,171 @@ class Page(MPTTModel, TitleSlugMixin, HasContentMixin):
         for p in self.get_descendants():
             exclude_list.append(p.id)
         return Page.objects.exclude(id__in=exclude_list)
+    
+    ### Content methods
+    
+    def get_content(self, language, ctype, language_fallback=False):
+        """Shortcut method for retrieving a piece of page content
+
+        :param language: wanted language, if not defined default is used.
+        :param ctype: the type of content.
+        :param fallback: if ``True``, the content will also be searched in \
+        other languages.
+        """
+        return Content.objects.get_content(self, language, ctype,
+            language_fallback)
+
+    def expose_content(self):
+        """Return all the current content of this page into a `string`.
+
+        This is used by the haystack framework to build the search index."""
+        placeholders = get_placeholders(self.get_template())
+        exposed_content = []
+        for lang in self.get_languages():
+            for ctype in [p.name for p in placeholders]:
+                content = self.get_content(lang, ctype, False)
+                if content:
+                    exposed_content.append(content)
+        return "\r\n".join(exposed_content)
+
+    def content_by_language(self, language):
+        """
+        Return a list of latest published
+        :class:`Content <pages.models.Content>`
+        for a particluar language.
+
+        :param language: wanted language,
+        """
+        placeholders = get_placeholders(self.get_template())
+        content_list = []
+        for ctype in [p.name for p in placeholders]:
+            try:
+                content = Content.objects.get_content_object(self,
+                    language, ctype)
+                content_list.append(content)
+            except Content.DoesNotExist:
+                pass
+        return content_list
+        
+    ### Title and slug
+    
+    def get_url_path(self, language=None):
+        """Return the URL's path component. Add the language prefix if
+        ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to ``True``.
+
+        :param language: the wanted url language.
+        """
+        if self.is_first_root():
+            # this is used to allow users to change URL of the root
+            # page. The language prefix is not usable here.
+            try:
+                return reverse('pages-root')
+            except Exception:
+                pass
+        url = self.get_complete_slug(language)
+        if not language:
+            language = settings.PAGE_DEFAULT_LANGUAGE
+        if settings.PAGE_USE_LANGUAGE_PREFIX:
+            return reverse('pages-details-by-path',
+                args=[language, url])
+        else:
+            return reverse('pages-details-by-path', args=[url])
+
+    def get_absolute_url(self, language=None):
+        """Alias for `get_url_path`.
+        
+        :param language: the wanted url language.
+        """
+        return self.get_url_path(language=language)
+
+    def get_complete_slug(self, language=None, hideroot=True):
+        """Return the complete slug of this page by concatenating
+        all parent's slugs.
+
+        :param language: the wanted slug language."""
+        if not language:
+            language = settings.PAGE_DEFAULT_LANGUAGE
+
+        if self._complete_slug and language in self._complete_slug:
+            return self._complete_slug[language]
+
+        self._complete_slug = cache.get(self.PAGE_URL_KEY % (self.id))
+        if self._complete_slug is None:
+            self._complete_slug = {}
+        elif language in self._complete_slug:
+            return self._complete_slug[language]
+
+        if hideroot and settings.PAGE_HIDE_ROOT_SLUG and self.is_first_root():
+            url = ''
+        else:
+            url = '%s' % self.slug(language)
+
+        key = self.ANCESTORS_KEY % self.id
+        ancestors = cache.get(key, None)
+        if ancestors is None:
+            ancestors = self.get_ancestors(ascending=True)
+            cache.set(key, ancestors)
+
+        for ancestor in ancestors:
+            url = ancestor.slug(language) + '/' + url
+
+        self._complete_slug[language] = url
+        cache.set(self.PAGE_URL_KEY % (self.id), self._complete_slug)
+        return url
+
+    def slug_with_level(self, language=None):
+        """Display the slug of the page prepended with insecable
+        spaces to simluate the level of page in the hierarchy."""
+        level = ''
+        if self.level:
+            for n in range(0, self.level):
+                level += '&nbsp;&nbsp;&nbsp;'
+        return mark_safe(level + self.slug(language))
+
+    def slug(self, language=None, fallback=True):
+        """
+        Return the slug of the page depending on the given language.
+
+        :param language: wanted language, if not defined default is used.
+        :param fallback: if ``True``, the slug will also be searched in other \
+        languages.
+        """
+
+        slug = self.get_content(language, 'slug', language_fallback=fallback)
+
+        return slug
+
+    def title(self, language=None, fallback=True):
+        """
+        Return the title of the page depending on the given language.
+
+        :param language: wanted language, if not defined default is used.
+        :param fallback: if ``True``, the slug will also be searched in \
+        other languages.
+        """
+        if not language:
+            language = settings.PAGE_DEFAULT_LANGUAGE
+
+        return self.get_content(language, 'title', language_fallback=fallback)
+        
+    ### Formating methods
 
     def margin_level(self):
         """Used in the admin menu to create the left margin."""
         return self.level * 2
 
-    def dump_json_data(self):
-        """
-        Return a python dict representation of this page for use as part of
-        a JSON export.
-        """
-        def content_langs_ordered():
-            """
-            Return a list of languages ordered by the page content
-            with the latest creation date in each.  This will be used
-            to maintain the state of the language_up_to_date template
-            tag when a page is restored or imported into another site.
-            """
-            params = {'page': self}
-            if self.freeze_date:
-                params['creation_date__lte'] = self.freeze_date
-            cqs = Content.objects.filter(**params)
-            cqs = cqs.values('language').annotate(latest=Max('creation_date'))
-            return [c['language'] for c in cqs.order_by('latest')]
-        languages = content_langs_ordered()
-
-        def language_content(ctype):
-            return dict(
-                (lang, self.get_content(lang, ctype, language_fallback=False))
-                for lang in languages)
-
-        def placeholder_content():
-            """Return content of each placeholder in each language."""
-            out = {}
-            for p in get_placeholders(self.get_template()):
-                if p.name in ('title', 'slug'):
-                    continue # these were already included
-                out[p.name] = language_content(p.name)
-            return out
-
-        def isoformat(d):
-            return None if d is None else d.strftime(ISODATE_FORMAT)
-
-        def custom_email(user):
-            """Allow a user's profile to return an email for the user."""
-            try:
-                profile = user.get_profile()
-            except (SiteProfileNotAvailable, ObjectDoesNotExist):
-                return user.email
-            get_email = getattr(profile, 'get_email', None)
-            return get_email() if get_email else user.email
-
-        return {
-            'complete_slug': dict(
-                (lang, self.get_complete_slug(lang, hideroot=False))
-                for lang in languages),
-            'title': language_content('title'),
-            'author_email': custom_email(self.author),
-            'creation_date': isoformat(self.creation_date),
-            'publication_date': isoformat(self.publication_date),
-            'publication_end_date': isoformat(self.publication_end_date),
-            'last_modification_date': isoformat(self.last_modification_date),
-            'status': {
-                Page.PUBLISHED: 'published',
-                Page.HIDDEN: 'hidden',
-                Page.DRAFT: 'draft'}[self.status],
-            'template': self.template,
-            'sites': (
-                [site.domain for site in self.sites.all()]
-                if settings.PAGE_USE_SITE_ID else []),
-            'redirect_to_url': self.redirect_to_url,
-            'redirect_to_complete_slug': dict(
-                (lang, self.redirect_to.get_complete_slug(
-                    lang, hideroot=False))
-                for lang in self.redirect_to.get_languages()
-                ) if self.redirect_to is not None else None,
-            'content': placeholder_content(),
-            'content_language_updated_order': languages,
-        }
-
-    def update_redirect_to_from_json(self, redirect_to_complete_slugs):
-        """
-        The second pass of PageManager.create_and_update_from_json_data
-        used to update the redirect_to field.
-
-        Returns a messages list to be appended to the messages from the
-        first pass.
-        """
-        messages = []
-        s = ''
-        for lang, s in redirect_to_complete_slugs.items():
-            r = Page.objects.from_path(s, lang, exclude_drafts=False)
-            if r:
-                self.redirect_to = r
-                self.save()
-                break
-        else:
-            messages.append(_("Could not find page for redirect-to field"
-                " '%s'") % (s,))
-        return messages
-
-    def __unicode__(self):
+    def __str__(self):
         """Representation of the page, saved or not."""
         if self.id:
             # without ID a slug cannot be retrieved
             slug = self.slug()
             if slug:
                 return slug
-            return u"Page %d" % self.id
-        return u"Page without id"
+            return "Page {0}".format(self.id)
+        return "Page without id"
 
 
+@python_2_unicode_compatible
 class Content(models.Model):
     """A block of content, tied to a :class:`Page <pages.models.Page>`,
     for a particular language"""
@@ -647,10 +511,10 @@ class Content(models.Model):
         verbose_name = _('content')
         verbose_name_plural = _('contents')
 
-    def __unicode__(self):
-        return u"%s :: %s" % (self.page.slug(), self.body[0:15])
-
+    def __str__(self):
+        return "{0} :: {1}".format(self.page.slug(), self.body[0:15])
 
+@python_2_unicode_compatible
 class PageAlias(models.Model):
     """URL alias for a :class:`Page <pages.models.Page>`"""
     page = models.ForeignKey(Page, null=True, blank=True,
@@ -666,6 +530,6 @@ class PageAlias(models.Model):
         self.url = normalize_url(self.url)
         super(PageAlias, self).save(*args, **kwargs)
 
-    def __unicode__(self):
-        return u"%s :: %s" % (self.url, self.page.get_complete_slug())
+    def __str__(self):
+        return "{0} :: {1}".format(self.url, self.page.get_complete_slug())
 
