@@ -1,5 +1,4 @@
 """Placeholder module, that's where the smart things happen."""
-
 from pages.widgets_registry import get_widget
 from pages import settings
 from pages.models import Content
@@ -18,7 +17,7 @@ from django.utils.safestring import mark_safe
 from django.utils.text import unescape_string_literal
 from django.template.loader import render_to_string
 from django.template import RequestContext
-
+from django.core.files.uploadedfile import UploadedFile
 import logging
 import os
 import time
@@ -250,7 +249,7 @@ def get_filename(page, placeholder, data):
     filename = os.path.join(
         settings.PAGE_UPLOAD_ROOT,
         'page_' + str(page.id),
-        placeholder.name + '-' + str(time.time()) + '-' + str(data)
+        placeholder.name + '-' + str(time.time()) + '-' + data.name
     )
     return filename
 
@@ -282,7 +281,7 @@ class ImagePlaceholderNode(PlaceholderNode):
         filename = ''
         if change and data:
             # the image URL is posted if not changed
-            if type(data) is str:
+            if not isinstance(data, UploadedFile):
                 return
 
             filename = get_filename(page, self, data)
@@ -321,7 +320,7 @@ class FilePlaceholderNode(PlaceholderNode):
         filename = ''
         if change and data:
             # the image URL is posted if not changed
-            if type(data) is str:
+            if not isinstance(data, UploadedFile):
                 return
 
             filename = get_filename(page, self, data)
@@ -343,7 +342,7 @@ class ContactForm(forms.Form):
       max_length=150)
     message = forms.CharField(widget=forms.Textarea(),
       label=_('Your message'))
-    
+
 
 class ContactPlaceholderNode(PlaceholderNode):
     """A contact `PlaceholderNode` example."""
