@@ -1,17 +1,16 @@
 """Page CMS page_tags template tags"""
 from django import template
-from django.utils.safestring import SafeUnicode
+from django.utils.safestring import SafeText
 from django.template import TemplateSyntaxError
-#from django.forms import Widget, Textarea, ImageField, CharField
-import urllib
 from django.conf import settings
 
 from pages import settings as pages_settings
 from pages.models import Content, Page
 from pages.placeholders import PlaceholderNode, ImagePlaceholderNode, FilePlaceholderNode
-from pages.placeholders import VideoPlaceholderNode, ContactPlaceholderNode
+from pages.placeholders import ContactPlaceholderNode, MarkdownPlaceholderNode
 from pages.placeholders import JsonPlaceholderNode, parse_placeholder
-
+from six.moves import urllib
+import six
 
 register = template.Library()
 
@@ -21,8 +20,8 @@ def get_page_from_string_or_id(page_string, lang=None):
     if type(page_string) == int:
         return Page.objects.get(pk=int(page_string))
     # if we have a string coming from some templates templates
-    if (isinstance(page_string, SafeUnicode) or
-        isinstance(page_string, unicode)):
+    if (isinstance(page_string, SafeText) or
+        isinstance(page_string, six.string_types)):
         if page_string.isdigit():
             return Page.objects.get(pk=int(page_string))
         return Page.objects.from_path(page_string, lang)
@@ -126,10 +125,10 @@ def pages_admin_menu(context, page):
 
     expanded = False
     if request and "tree_expanded" in request.COOKIES:
-        cookie_string = urllib.unquote(request.COOKIES['tree_expanded'])
+        cookie_string = urllib.parse.unquote(request.COOKIES['tree_expanded'])
         if cookie_string:
             ids = [int(id) for id in
-                urllib.unquote(request.COOKIES['tree_expanded']).split(',')]
+                urllib.parse.unquote(request.COOKIES['tree_expanded']).split(',')]
             if page.id in ids:
                 expanded = True
     context.update({'expanded': expanded, 'page': page})
@@ -165,22 +164,9 @@ show_content = register.inclusion_tag('pages/content.html',
                                       takes_context=True)(show_content)
 
 
-def show_slug_with_level(context, page, lang=None, fallback=True):
-    """Display slug with level by language."""
-    if not lang:
-        lang = context.get('lang', pages_settings.PAGE_DEFAULT_LANGUAGE)
-
-    page = get_page_from_string_or_id(page, lang)
-    if not page:
-        return ''
-
-    return {'content': page.slug_with_level(lang)}
-show_slug_with_level = register.inclusion_tag('pages/content.html',
-                                      takes_context=True)(show_slug_with_level)
-
-
 def show_absolute_url(context, page, lang=None):
-    """Show the url of a page in the right language
+    """
+    Show the url of a page in the right language
 
     Example ::
 
@@ -192,7 +178,7 @@ def show_absolute_url(context, page, lang=None):
 
     Keyword arguments:
     :param page: the page object, slug or id
-    :param lang: the wanted language
+    :param lang: the wanted language \
         (defaults to `settings.PAGE_DEFAULT_LANGUAGE`)
     """
     if not lang:
@@ -436,6 +422,13 @@ def do_placeholder(parser, token):
     return PlaceholderNode(name, **params)
 register.tag('placeholder', do_placeholder)
 
+def do_markdownlaceholder(parser, token):
+    """
+    Method that parse the markdownplaceholder template tag.
+    """
+    name, params = parse_placeholder(parser, token)
+    return MarkdownPlaceholderNode(name, **params)
+register.tag('markdownplaceholder', do_markdownlaceholder)
 
 def do_imageplaceholder(parser, token):
     """
@@ -453,14 +446,6 @@ def do_fileplaceholder(parser, token):
     return FilePlaceholderNode(name, **params)
 register.tag('fileplaceholder', do_fileplaceholder)
 
-def do_videoplaceholder(parser, token):
-    """
-    Method that parse the imageplaceholder template tag.
-    """
-    name, params = parse_placeholder(parser, token)
-    return VideoPlaceholderNode(name, **params)
-register.tag('videoplaceholder', do_videoplaceholder)
-
 def do_contactplaceholder(parser, token):
     """
     Method that parse the contactplaceholder template tag.
