@@ -17,7 +17,7 @@ def suite():
         from pages.tests.test_selenium import SeleniumTestCase
         suite.addTest(unittest.makeSuite(SeleniumTestCase))
     except ImportError:
-        print("Install selenium and PhantomJS to run browsers tests")
+        print("You need to install selenium and PhantomJS to run browsers tests")
     # suite.addTest(unittest.makeSuite(LinkTestCase))
     # suite.addTest(unittest.makeSuite(AutoRenderTestCase))
     # being the slower test I run it at the end
